@@ -3,9 +3,6 @@
 -}
 
 -- | This is the module's Haddock.
-module Main
-  ( main,
-  )
-where
+module Main (main) where
 
 main = return ()
