@@ -1,9 +1,6 @@
 module Test
   {-# DEPRECATED "This module is unstable" #-}
-  ( foo,
-    bar,
-    baz,
-  )
+  (foo, bar, baz)
 where
 
 import Blah
